import matplotlib.pyplot as plt
import numpy as np
import matplotlib.colors as colors
from matplotlib.gridspec import GridSpec
from matplotlib import ticker
from matplotlib.collections import LineCollection
import matplotlib.image as mpimg

# Set font parameters
plt.rcParams.update({
    'backend': 'TkAgg',
    'font.family': 'sans-serif',
    'font.sans-serif': ['DejaVu Sans', 'Helvetica', 'Bitstream Vera Sans', 'Arial', 'sans-serif'],
    'font.size': 11,
    'xtick.labelsize': 11,
    'ytick.labelsize': 11,
    'axes.linewidth': 0.8,
    'xtick.major.width': 1.0,
    'ytick.major.width': 1.0,
    'legend.columnspacing': 0.5,
    'legend.handletextpad': 0.25,
    'legend.borderaxespad': 0.00,
})

# Create main figure
fig = plt.figure(figsize=(12, 3))

# Panel labels
labels = ['(a)', '(b)', '(c)', '(d)']

# Define exact positions for each panel [left, bottom, width, height]
struct_pos = [0.00, -0.04, 0.15, 0.91]  # Panel (a)
band_pos = [0.17, 0.10, 0.20, 0.70*1.08]    # Panel (b) - band structure, increased height by 10%
dos_pos = [0.385, 0.10, 0.07, 0.70*1.08]     # Panel (b) - DOS, increased height by 10%
bz_pos = [0.456, 0.00, 0.23, 0.9]      # Panel (c)
weyl_pos = [0.74, 0.08, 0.25, 0.91]    # Panel (d)

# Define Fermi energy and other parameters
Ef = 17.063  # Fermi energy
xmin, xmax = 0.0, 4.2424
ymin, ymax = -2.0, 2.0

# Create axes using exact positions
ax_struct = fig.add_axes(struct_pos)
ax_bands = fig.add_axes(band_pos)
ax_dos = fig.add_axes(dos_pos)  # Removed sharey parameter
ax_dos.set_ylim(ymin, ymax)  # Set the same y-limits manually
ax_bz = fig.add_axes(bz_pos)

# Position for colorbar above band structure - increased height and reduced width
cbar_pos = [band_pos[0] + band_pos[2]*0.1, band_pos[1] + band_pos[3] + 0.02, band_pos[2]*0.8, 0.04]
cbar_ax = fig.add_axes(cbar_pos)

# Add panel labels
ax_struct.text(0.05, 1.02, labels[0], transform=ax_struct.transAxes, 
              fontsize=12, fontweight='normal')
ax_bands.text(-0.2, 1.045, labels[1], transform=ax_bands.transAxes, 
             fontsize=12, fontweight='normal')
ax_bz.text(0.02, 0.99, labels[2], transform=ax_bz.transAxes, 
           fontsize=12, fontweight='normal')

#========================
# (a) Crystal Structure
#========================
try:
    # Load and display structure image
    struct_img = mpimg.imread('struct.png')
    ax_struct.imshow(struct_img)
    ax_struct.axis('off')
except FileNotFoundError:
    ax_struct.text(0.5, 0.5, 'struct.png not found', 
                 ha='center', va='center', transform=ax_struct.transAxes)

#========================
# (b) Band Structure + DOS
#========================
# Function to process and reshape DFT data
def process_bands(band_file):
    try:
        data = np.loadtxt(band_file)
        k = np.unique(data[:, 0])
        num_bands = len(data) // len(k)
        bands = np.reshape(data[:, 1], (num_bands, len(k)))
        weight = np.reshape(data[:, 2], (num_bands, len(k)))
        return k, bands, weight
    except FileNotFoundError:
        print(f"Warning: {band_file} not found.")

# Create circular pattern of annotation points around each Weyl node
def create_circular_annotation(ax, center_x, center_y, radius=1.5, points=8):
    for i in range(points):
        angle = 2 * np.pi * i / points
        x = center_x + radius * np.cos(angle)
        y = center_y + radius * np.sin(angle)
        ax.plot([center_x, x], [center_y, y], 'k-', lw=1.0, alpha=0.8, zorder=20)

# Try to load DOS data
try:
    total_dos = np.loadtxt('TaAs_dos.dat', skiprows=1)
    ta_dos = np.loadtxt('Ta_dos.dat', skiprows=1)
    as_dos = np.loadtxt('As_dos.dat', skiprows=1)
    dos_loaded = True
except FileNotFoundError:
    print("Warning: DOS files not found.")

# Try to load band structure data
k, bands, weight = process_bands('fatband_TaAs.dat')

# Define x-coordinates from gnuplot script
x1, x2, x3, x4, x5, x6, x7 = 0.0000, 0.9984, 1.2808, 1.5632, 2.4018, 2.9435, 4.2424

# Band structure plot settings
ax_bands.set_xlim(xmin, xmax)
ax_bands.set_ylim(ymin, ymax)

# Add vertical lines at high-symmetry points
for x in [x2, x3, x4, x5, x6]:
    ax_bands.axvline(x=x, color='black', linestyle='-', linewidth=0.2, alpha=0.5, zorder=0)

# Add horizontal line at zero energy
ax_bands.axhline(y=0, color='black', linestyle='--', linewidth=0.5, alpha=0.5, zorder=0)

# Set x-ticks at high-symmetry points
ax_bands.set_xticks([xmin, x2, x3, x4, x5, x6, xmax])
ax_bands.set_xticklabels([r'$\Gamma$', r'$\Sigma$', 'N', r'$\Sigma_1$', 'Z', r'$\Gamma$', 'X'])
ax_bands.yaxis.set_minor_locator(ticker.AutoMinorLocator(4))

# Add y-label with a larger pad to make it more visible
ax_bands.set_ylabel(r"$\varepsilon - \varepsilon_{\rm F}$ (eV)", labelpad=2, fontsize=12)

# Create custom colormap
cmap = colors.LinearSegmentedColormap.from_list('custom_cmap', ['red', 'blue'])
norm = colors.Normalize(vmin=0, vmax=0.5)

# Plot bands with color according to weights
for i in range(bands.shape[0]):
    points = np.array([k, bands[i]]).T.reshape(-1, 1, 2)
    segments = np.concatenate([points[:-1], points[1:]], axis=1)
    lc = LineCollection(segments, cmap=cmap, norm=norm, linewidth=1.0)
    lc.set_array(weight[i, :-1])  # Use weights for coloring
    ax_bands.add_collection(lc)

# Create colorbar
colorbar = plt.colorbar(plt.cm.ScalarMappable(norm=norm, cmap=cmap), 
                        cax=cbar_ax, orientation='horizontal')
colorbar.set_ticks([])  # Remove ticks

# Move labels closer to colorbar
cbar_ax.text(-0.05, 0.5, 'As', ha='right', va='center', color='black', fontsize=10, transform=cbar_ax.transAxes)
cbar_ax.text(1.05, 0.5, 'Ta', ha='left', va='center', color='black', fontsize=10, transform=cbar_ax.transAxes)

# DOS plot settings
ax_dos.set_xlim(0, 7.5)
ax_dos.set_xlabel('DOS\n(States/eV)')
ax_dos.set_xticks([0, 5])
ax_dos.yaxis.set_minor_locator(ticker.AutoMinorLocator(4))
ax_dos.xaxis.set_minor_locator(ticker.AutoMinorLocator(2))
ax_dos.axhline(y=0, color='black', linestyle='--', linewidth=0.5, alpha=0.5, zorder=0)
ax_dos.set_yticklabels([])  # Hide y-ticks

# Plot DOS data
if dos_loaded:
    ax_dos.fill(total_dos[:, 1], total_dos[:, 0]-Ef, color='gray', alpha=0.2, label='Total')
    ax_dos.plot(ta_dos[:, 1], ta_dos[:, 0]-Ef, 'b-', linewidth=1.0, label='Ta')
    ax_dos.plot(as_dos[:, 1], as_dos[:, 0]-Ef, 'r-', linewidth=1.0, label='As')
    ax_dos.legend(loc='upper right', fontsize=10, frameon=False, handlelength=0.8)

#========================
# (c) Brillouin Zone with Weyl Points
#========================
# Load and display BZ-weyl image
try:
    bz_img = mpimg.imread('BZ-weyl.png')
    ax_bz.imshow(bz_img, aspect='auto')
    ax_bz.axis('off')
except FileNotFoundError:
    ax_bz.text(0.5, 0.5, 'BZ-weyl.png not found', 
              ha='center', va='center', transform=ax_bz.transAxes)

#========================
# (d) Weyl Points
#========================
# Define Weyl point subplot positions with increased spacing
weyl_width = weyl_pos[2] / 3.6  # Reduced width to create horizontal space
weyl_height = weyl_pos[3] / 2.55  # Reduced height to create vertical space
weyl_left = weyl_pos[0]
weyl_bottom = weyl_pos[1]

# Set common parameters for x-axis scaling
scale_factor = 0.05  # This is the factor to multiply by 2π/a


# Create individual Weyl point subplots with more spacing
w1_kx_pos = [weyl_left, weyl_bottom + weyl_height + 0.08, weyl_width, weyl_height]  # Increased vertical gap
w1_ky_pos = [weyl_left + weyl_width * 1.4, weyl_bottom + weyl_height + 0.08, weyl_width, weyl_height]  # Added horizontal spacing
w1_kz_pos = [weyl_left + 2.8*weyl_width, weyl_bottom + weyl_height + 0.08, weyl_width, weyl_height]  # Added horizontal spacing
w2_kx_pos = [weyl_left, weyl_bottom, weyl_width, weyl_height]
w2_ky_pos = [weyl_left + weyl_width * 1.4, weyl_bottom, weyl_width, weyl_height]  # Added horizontal spacing
w2_kz_pos = [weyl_left + 2.8*weyl_width, weyl_bottom, weyl_width, weyl_height]  # Added horizontal spacing

ax_w1_kx = fig.add_axes(w1_kx_pos)
ax_w1_ky = fig.add_axes(w1_ky_pos)
ax_w1_kz = fig.add_axes(w1_kz_pos)
ax_w2_kx = fig.add_axes(w2_kx_pos)
ax_w2_ky = fig.add_axes(w2_ky_pos)
ax_w2_kz = fig.add_axes(w2_kz_pos)

# Add panel label
ax_w1_kx.text(-0.58, 1.03, labels[3], transform=ax_w1_kx.transAxes, 
              fontsize=12, fontweight='normal')

# Function to read band data
def read_bands_separated_by_blanklines(filename, x_offset=0.0, Ef=0.0):
    try:
        bands = []
        current_band = []
        with open(filename, 'r') as f:
            for line in f:
                if line.strip() == "":
                    if current_band:
                        bands.append(np.array(current_band))
                        current_band = []
                else:
                    parts = line.strip().split()
                    if len(parts) >= 2:
                        x = float(parts[0]) - x_offset
                        y = float(parts[1]) - Ef
                        current_band.append([x, y])
            if current_band:
                bands.append(np.array(current_band))
        return bands
    except FileNotFoundError:
        print(f"Warning: {filename} not found")

# Weyl point configurations
Ef_weyl = 17.0633
ymin_weyl, ymax_weyl = -30, 30

# Define Weyl node positions (x, y coordinates) for each plot
weyl_nodes = {
    'w1_kx': [0, -8.5],     
    'w1_ky': [-0.015, -8.5], 
    'w1_ky2': [0.015, -8.5],
    'w1_kz': [0, -8.5],     
    'w2_kx': [-0.035, 2.5],     
    'w2_kx2': [0.035, 2.5],      
    'w2_ky': [0, 2.5],      
    'w2_kz': [0, 2.5]       
}

configs = [
    # Top row (W1)
    {"file": "new_bands_w1_kx.dat", "xlabel": r"$k_x$", "x_offset": 0.0500, "xlen": 0.1, "title": None, "ylabel": "Energy (meV)",
     "nodes": ["w1_kx"]},
    {"file": "new_bands_w1_ky.dat", "xlabel": r"$k_y$", "x_offset": 0.0000, "xlen": 0.1, "title": "W1", "ylabel": None,
     "nodes": ["w1_ky", "w1_ky2"]},
    {"file": "new_bands_w1_kz.dat", "xlabel": r"$k_z$", "x_offset": 0.0000, "xlen": 0.1, "title": None, "ylabel": None,
     "nodes": ["w1_kz"]},
    # Bottom row (W2)
    {"file": "new_bands_w2_kx.dat", "xlabel": r"$k_x$", "x_offset": 0.0, "xlen": 0.1, "title": None, "ylabel": None,
     "nodes": ["w2_kx", "w2_kx2"]},  # Added w2_kx2
    {"file": "new_bands_w2_ky.dat", "xlabel": r"$k_y$", "x_offset": -0.517599, "xlen": 0.1, "title": "W2", "ylabel": None,
     "nodes": ["w2_ky"]},
    {"file": "new_bands_w2_kz.dat", "xlabel": r"$k_z$", "x_offset": 0.320322, "xlen": 0.1, "title": None, "ylabel": None,
     "nodes": ["w2_kz"]},
]
axes_weyl = [ax_w1_kx, ax_w1_ky, ax_w1_kz, ax_w2_kx, ax_w2_ky, ax_w2_kz]

# Plot loop for Weyl points
for ax, cfg in zip(axes_weyl, configs):
    bands = read_bands_separated_by_blanklines(cfg["file"], x_offset=cfg["x_offset"], Ef=Ef_weyl)
    
    # Plot bands
    for band in bands:
        x, y = band[:, 0], band[:, 1]*1000  # Convert to meV
        if ax in [ax_w1_kx, ax_w1_ky, ax_w1_kz]:
            ax.plot(x, y, color='r', linewidth=1.2)
        else:
            ax.plot(x, y, color='b', linewidth=1.2)
    
    weyl_energy = -8 if ax in [ax_w1_kx, ax_w1_ky, ax_w1_kz] else 3
    ax.axhline(weyl_energy, color='black', linestyle=':', linewidth=1.5, alpha=0.9)
    
    ax.axhline(0, color='black', linestyle='--', linewidth=0.6, alpha=0.6, zorder=0)
    ax.axvline(0, color='black', linestyle='--', linewidth=0.6, alpha=0.6, zorder=0)

    for node_key in cfg["nodes"]:
        node_x, node_y = weyl_nodes[node_key]
        ax.plot(node_x, node_y, 'o', markersize=7, markeredgecolor='black', 
                markerfacecolor='none', markeredgewidth=1.0, zorder=10)

    xlen = cfg["xlen"]
    ax.set_xlim([-xlen / 2, xlen / 2])
    ax.set_ylim([ymin_weyl, ymax_weyl])
    ax.set_xticks([-0.05, 0, 0.05])
    ax.set_xticklabels(['-0.1', '0', '0.1'])
    ax.xaxis.set_minor_locator(ticker.AutoMinorLocator(4))
    ax.yaxis.set_minor_locator(ticker.AutoMinorLocator(4))
    
    if ax in [ax_w2_kx, ax_w2_ky, ax_w2_kz]:
        ax.set_xlabel(cfg["xlabel"], labelpad=2)
    else:
        ax.set_xticklabels([])
    
    if ax in [ax_w1_ky, ax_w1_kz, ax_w2_ky, ax_w2_kz]:
        ax.set_yticklabels([])
    
fig.text(weyl_left + weyl_pos[2]/2, weyl_bottom + weyl_height*2 + 0.1, 'W1', 
         ha='center', va='center')
fig.text(weyl_left + weyl_pos[2]/2, weyl_bottom + weyl_height+0.025, 'W2', 
         ha='center', va='center')

unit_note = r'$(\times 0.5 \cdot 2\pi/a_0)$'

fig.text(weyl_left + weyl_pos[2]/2, weyl_bottom - 0.18, unit_note, 
         ha='center', va='center')


fig.text(weyl_left-0.05, weyl_bottom + weyl_pos[3]/2, 
         r'$\varepsilon - \varepsilon_{\rm F}$ (meV)', va='center', rotation='vertical', fontsize=12)

plt.savefig('Fig1.png', dpi=300, bbox_inches='tight')
plt.savefig('Fig1.pdf', bbox_inches='tight')
plt.show()