import matplotlib.pyplot as plt
import numpy as np
import matplotlib.gridspec as gridspec
from matplotlib.colors import LinearSegmentedColormap, Normalize, LogNorm
from matplotlib import ticker
from matplotlib.collections import LineCollection
import matplotlib.colors as mcolors
from matplotlib.ticker import LogFormatter, LogLocator
from matplotlib.ticker import FuncFormatter
from matplotlib.offsetbox import OffsetImage, AnnotationBbox
import matplotlib.image as mpimg

# Define formatter functions at the top level
def log_formatter(x, pos):
    """Custom formatter to display ticks as 10^n instead of 1e+n"""
    exponent = int(np.log10(x))
    return r'$10^{%d}$' % exponent

def better_log_formatter(x, pos):
    """Custom formatter to display ticks as 10^n with proper LaTeX"""
    if x == 0:
        return r'$0$'
    exponent = int(np.log10(x))
    return r'$10^{%d}$' % exponent

# Function to process and reshape DFT data for the phonon dispersion plot
def process_bands(band_file):
    print(f"Reading band data from: {band_file}")
    # Try loading with different column indices to handle various file formats
    data = np.loadtxt(band_file, skiprows=1)
    k = np.unique(data[:, 0])
    num_bands = len(data) // len(k)
    bands = np.reshape(data[:, 1], (num_bands, len(k)))
    
    # Try different weight columns (3 or 2) - this is where our issue might be
    if data.shape[1] > 3:
        weight = np.reshape(data[:, 3], (num_bands, len(k)))
        print("Using column 4 for weights")
    elif data.shape[1] > 2:
        weight = np.reshape(data[:, 2], (num_bands, len(k)))
        print("Using column 3 for weights")
    else:
        weight = np.ones_like(bands)
        print("No weight column found, using default weights")        
    return k, bands, weight

# Function to process and reshape data for the phonon scattering plot
def process_scattering_data(file_path):
    print(f"Reading scattering data from: {file_path}")
    try:
        # Load the data
        data = np.loadtxt(file_path, skiprows=1)
        k = np.unique(data[:, 0])
        
        # Determine number of bands based on the number of k-points
        num_k = len(k)
        num_bands = len(data) // num_k
        
        # Reshape the data into bands
        frequencies = np.reshape(data[:, 1], (num_bands, num_k))
        couplings_all = np.reshape(data[:, 2], (num_bands, num_k))
        couplings = np.copy(couplings_all)
        couplings[couplings <= 0] = 1e-10
        
        return k, frequencies, couplings
    except Exception as e:
        print(f"Error processing scattering data: {e}")
        return None, None, None

# Function to load nesting function data
def load_nesting_function(file_path):
    print(f"Reading nesting function data from: {file_path}")
    data = np.loadtxt(file_path, skiprows=1)
    # The fourth column divided by scaling factor 
    x_data = data[:, 0]
    y_data = data[:, 3] / 185.096025
    return x_data, y_data

# Function to read phdos data
def read_phdos_data(phdos_file, num_ta, num_as):
    print(f"Reading phdos data from: {phdos_file}")
    data = np.loadtxt(phdos_file, skiprows=1, unpack=True)
    #convert freq from cm-1 to meV
    freq = data[0] * 0.123983
    total_phdos = data[1]
    Ta_phdos = np.sum(data[2:num_ta+2], axis=0)
    As_phdos = np.sum(data[num_ta+2:num_ta+2+num_as], axis=0)
    return freq, total_phdos, Ta_phdos, As_phdos

# Function to read Raman experimental data
def read_raman_data(file_path):
    print(f"Reading experimental Raman data from: {file_path}")
    try:
        # Read the file, skipping comment lines
        with open(file_path, 'r') as f:
            lines = [line.strip() for line in f if line.strip() and not line.strip().startswith('#')]
            
        modes = []
        cm1_values = []
        meV_values = []
        
        for line in lines:
            parts = line.split('\t')
            if len(parts) >= 3:
                modes.append(parts[0])
                cm1_values.append(float(parts[1]))
                meV_values.append(float(parts[2]))
        return modes, cm1_values, meV_values
    except Exception as e:
        print(f"Error reading Raman data: {e}")
        return [], [], []

# Set font parameters
plt.rcParams.update({
    'font.family': 'sans-serif',
    'font.sans-serif': ['DejaVu Sans', 'Helvetica', 'Bitstream Vera Sans', 'Arial', 'sans-serif'],
    'font.size': 10,  
    'xtick.labelsize': 10,
    'ytick.labelsize': 10,
    'axes.linewidth': 0.8,
    'xtick.major.width': 1.0,
    'ytick.major.width': 1.0,
    'legend.columnspacing': 0.5,
    'legend.handletextpad': 0.25,
    'legend.borderaxespad': 0.00,
})

# Create a figure for the three-column layout with custom spacing
fig = plt.figure(figsize=(12.0, 3.0))

# Increase wspace between columns 1 and 2, but keep normal spacing between 2 and 3
gs = gridspec.GridSpec(1, 3, width_ratios=[1, 1, 1], wspace=0.1)  # Increased spacing

# =========== COLUMN 1: PHONON DISPERSION AND DOS ===========
colors_1 = [(0, 'black'), (0.5, 'yellow'), (1, 'red')]
custom_cmap_1 = LinearSegmentedColormap.from_list('custom_cmap_1', colors_1)

# Create gridspec for column 1 - MODIFIED HEIGHT RATIO FOR COLORBAR
gs1 = gridspec.GridSpecFromSubplotSpec(2, 2, 
                                      subplot_spec=gs[0], 
                                      height_ratios=[0.08, 1],  
                                      width_ratios=[0.75, 0.25], 
                                      wspace=0.10,  
                                      hspace=0.10) 

# Initialize subplots for column 1
ax_cb1 = fig.add_subplot(gs1[0, 0])  # Colorbar above
ax1 = fig.add_subplot(gs1[1, 0])     # Bands
ax2 = fig.add_subplot(gs1[1, 1], sharey=ax1)  # DOS

# Load phonon frequency data
k, bands, weight = process_bands("phfreq_g2.dat")
print(f"Band structure data loaded successfully: {bands.shape} bands")
    
# Define high-symmetry points
x1, x2, x3, x4, x5, x6, x7 = 0.0000, 0.9984, 1.2808, 1.5632, 2.4018, 2.9435, 4.2424
xmin, xmax = 0.0, 4.2424
symmetry_points = [x1, x2, x3, x4, x5, x6, x7]
labels = [r'$\Gamma$', r'$\Sigma$', 'N', r'$\Sigma_1$', 'Z', r'$\Gamma$', 'X']

# Set up band plot
ax1.set_xlim(xmin, xmax)
ax1.set_ylim(0, 35)

# Add vertical lines at symmetry points
for x in symmetry_points:
    ax1.axvline(x=x, color='black', linestyle='-', linewidth=0.2, alpha=0.5, zorder=0)

# Set axis labels and ticks
ax1.set_xticks(symmetry_points)
ax1.set_xticklabels(labels)
ax1.set_ylabel(r'$\omega$ (meV)')
ax1.set_yticks(np.arange(0, 36, 10))
ax1.yaxis.set_minor_locator(ticker.AutoMinorLocator(2))

# Create normalization for colors
norm1 = Normalize(vmin=0, vmax=15) 

# Apply the transformation to get g-values and plot bands
min_g = float('inf')
max_g = float('-inf')
for i in range(bands.shape[0]):
    # Create array of points for this band
    points = np.array([k, bands[i]]).T.reshape(-1, 1, 2)
    segments = np.concatenate([points[:-1], points[1:]], axis=1)
    
    # Calculate color values for this band
    g_values = np.sqrt(weight[i, :-1] * 13605) / 7
    
    # Keep track of min/max for debugging
    if len(g_values) > 0:
        if g_values.min() < min_g:
            min_g = g_values.min()
        if g_values.max() > max_g:
            max_g = g_values.max()
    
    # Create a line collection with color mapping
    lc = LineCollection(segments, cmap=custom_cmap_1, norm=norm1, linewidth=1.0)
    lc.set_array(g_values)
    ax1.add_collection(lc)
print(f"Overall g-value range: {min_g} to {max_g}")

# Load and plot experimental Raman data
modes, cm1_values, meV_values = read_raman_data("Raman_data.dat")

# Plot the experimental data at both Gamma points in the BZ
gamma_points = [x1, x6]  # Plot at both Gamma points
for gamma_x in gamma_points:
    # Plot each experimental point as an open circle
    for mode, meV in zip(modes, meV_values):
        ax1.plot(gamma_x, meV, 'o', markersize=4, markerfacecolor='none', 
                 markeredgecolor='k', markeredgewidth=1.0, zorder=1)

# Load DOS data
freq, total_phdos, Ta_phdos, As_phdos = read_phdos_data("dos.dat", 2, 2)

# Set up DOS plot
ax2.set_ylim(0, 35)  # Match the y-limit of ax1
ax2.set_xlim(0, 0.3)
ax2.set_xticks([0, 0.3])
ax2.set_xticklabels(['0', '0.3'])
ax2.set_xlabel('PhDOS\n(States/meV)')
ax2.xaxis.set_minor_locator(ticker.AutoMinorLocator(3))
ax2.yaxis.set_minor_locator(ticker.AutoMinorLocator(2))
# Hide only y-tick labels for DOS plot
plt.setp(ax2.get_yticklabels(), visible=False)

# Plot DOS data
ax2.fill(total_phdos, freq, color='gray', alpha=0.2, label='Total')
ax2.plot(Ta_phdos, freq, 'b-', linewidth=1.0, label='Ta')
ax2.plot(As_phdos, freq, 'r-', linewidth=1.0, label='As')

# Add legend
ax2.legend(loc='lower right', fontsize=9, frameon=False, handlelength=0.8)

# Create a horizontal colorbar for column 1
sm1 = plt.cm.ScalarMappable(norm=norm1, cmap=custom_cmap_1)
sm1.set_array([])  # Dummy array for the colorbar

# Create colorbar
cbar1 = plt.colorbar(sm1, cax=ax_cb1, orientation='horizontal')

# Keep ticks on top side with reduced spacing
cbar1.ax.xaxis.set_ticks_position('top')
cbar1.ax.tick_params(direction='in', labelsize=8, pad=1)
cbar1.ax.xaxis.set_label_position('bottom')

# Set the colorbar ticks
cbar1.set_ticks([0, 5, 10, 15])

# Add the "(a)" label to the top-left corner
ax1.text(-0.25, 1.15, '(a)', transform=ax1.transAxes, fontsize=12, va='bottom', ha='left')

# =========== COLUMN 2: SECOND PLOT WITH LOG SCALE COLORBAR ===========
gs2 = gridspec.GridSpecFromSubplotSpec(2, 1, 
                                      subplot_spec=gs[1], 
                                      height_ratios=[0.08, 1],  
                                      hspace=0.10)  

# Initialize subplots for column 2
ax_cb2 = fig.add_subplot(gs2[0, 0])  # Colorbar above
ax3 = fig.add_subplot(gs2[1, 0])     # Main plot

# Create custom colormap for column 2 - blue-yellow-red for log scale
colors_2 = [(0, 'blue'), (0.5, 'yellow'), (1, 'red')]
custom_cmap_2 = LinearSegmentedColormap.from_list('custom_cmap_2', colors_2)

# Set up the second plot
ax3.set_xlim(xmin, xmax)
ax3.set_ylim(0, 35)
ax3.text(-0.25, 1.15, '(b)', transform=ax3.transAxes, fontsize=12, va='bottom', ha='left')

# Add vertical lines at symmetry points
for x in symmetry_points:
    ax3.axvline(x=x, color='black', linestyle='-', linewidth=0.2, alpha=0.5, zorder=0)

# Set axis labels and ticks
ax3.set_xticks(symmetry_points)
ax3.set_xticklabels(labels)
ax3.set_ylabel(r'$\omega$ (meV)')
ax3.set_yticks(np.arange(0, 36, 10))
ax3.yaxis.set_minor_locator(ticker.AutoMinorLocator(2))

# Create log normalization for colors (second plot)
norm2 = LogNorm(vmin=0.00001, vmax=0.05)

# Load data for second plot
k_scat, freqs_scat, couplings_scat = process_scattering_data("scat.dat")

# Plot each band separately with the log scale colors
for i in range(freqs_scat.shape[0]):
    # Create array of points for this band
    points = np.array([k_scat, freqs_scat[i]]).T.reshape(-1, 1, 2)
    segments = np.concatenate([points[:-1], points[1:]], axis=1)
    
    # Create a line collection with color mapping
    lc = LineCollection(segments, cmap=custom_cmap_2, norm=norm2, linewidth=1.0)
    lc.set_array(couplings_scat[i, :-1])  # Set the color values
    ax3.add_collection(lc)

# Create a horizontal colorbar for column 2 with log scale
sm2 = plt.cm.ScalarMappable(norm=norm2, cmap=custom_cmap_2)
sm2.set_array([])
cbar2 = plt.colorbar(sm2, cax=ax_cb2, orientation='horizontal')

# Keep ticks on top side with reduced spacing
cbar2.ax.xaxis.set_ticks_position('top')
cbar2.ax.tick_params(direction='in', labelsize=8, pad=1)
cbar2.ax.xaxis.set_label_position('bottom')

cbar2.ax.xaxis.set_major_locator(LogLocator(base=10.0, numticks=4))
cbar2.ax.xaxis.set_major_formatter(FuncFormatter(better_log_formatter))

# =========== COLUMN 3: BRILLOUIN ZONE IMAGE (TOP) AND NESTING FUNCTION PLOT (BOTTOM) ===========
gs3 = gridspec.GridSpecFromSubplotSpec(3, 1, 
                                      subplot_spec=gs[2],
                                      height_ratios=[0.00, 0.8, 0.7],  
                                      hspace=0.00)  

# Initialize subplots for column 3
ax_empty = fig.add_subplot(gs3[0, 0])  # Empty space for alignment
ax_bz = fig.add_subplot(gs3[1, 0])     # Brillouin Zone image
ax_nf = fig.add_subplot(gs3[2, 0])     # Nesting function plot

ax_empty.set_visible(False)

# Add the labels to the subplots
ax_bz.text(-0.31, 0.90, '(c)', transform=ax_bz.transAxes, fontsize=12, va='bottom', ha='left')
ax_nf.text(-0.30, 0.94, '(d)', transform=ax_nf.transAxes, fontsize=12, va='bottom', ha='left')

# ========== BRILLOUIN ZONE IMAGE ==========
# Load the BZ image
bz_img = mpimg.imread('BZ.png')
ax_bz.imshow(bz_img)
ax_bz.axis('off')  # Turn off axis for the image
print("BZ.png loaded successfully")

# ========== NESTING FUNCTION PLOT ==========
nf_x, nf_y = load_nesting_function("nf.dat")

# Define y-axis range for nesting function
ymin_nf = 0.0
ymax_nf = 1.5

# Plot the nesting function
ax_nf.plot(nf_x, nf_y, 'k-', linewidth=1.0)

# Add vertical lines at symmetry points
for x in symmetry_points:
    ax_nf.axvline(x=x, color='black', linestyle='-', linewidth=0.2, alpha=0.5, zorder=0)

# Set up the plot formatting
ax_nf.set_xlim(xmin, xmax)
ax_nf.set_ylim(ymin_nf, ymax_nf)
ax_nf.set_xticks(symmetry_points)
ax_nf.set_xticklabels(labels)
ax_nf.set_ylabel(r'$\zeta_q$ (eV$^{-2}$)')
ax_nf.set_yticks(np.arange(0, ymax_nf+0.01, 0.5))  # Set y-ticks at intervals of 0.5
ax_nf.yaxis.set_minor_locator(ticker.AutoMinorLocator(2))

# Add q1 and q2 annotations to the nesting function plot
ax_nf.text(0.85, 0.11, r'$q_2$', transform=ax_nf.transAxes, fontsize=10, va='bottom', ha='left')
ax_nf.text(0.58, 0.17, r'$q_1$', transform=ax_nf.transAxes, fontsize=10, va='bottom', ha='left')

# Ensure the plot starts at exactly 0
ax_nf.margins(x=0, y=0)


def adjust_plot_dimensions():
    # Get the position of the plots in data coordinates
    pos1 = ax1.get_position()
    pos3 = ax3.get_position()
    pos_bz = ax_bz.get_position()
    pos_nf = ax_nf.get_position()
    
    # Also get colorbar positions
    pos_cb1 = ax_cb1.get_position()
    pos_cb2 = ax_cb2.get_position()
    
    # Use the first plot's width as reference
    reference_width = pos1.width
    
    # Adjust widths of plots to match
    if pos3.width != reference_width:
        new_pos3 = [pos3.x0, pos3.y0, reference_width, pos3.height]
        ax3.set_position(new_pos3)
    
    if pos_bz.width != reference_width:
        new_pos_bz = [pos_bz.x0, pos_bz.y0, reference_width, pos_bz.height]
        ax_bz.set_position(new_pos_bz)
        
    if pos_nf.width != reference_width:
        new_pos_nf = [pos_nf.x0, pos_nf.y0, reference_width, pos_nf.height]
        ax_nf.set_position(new_pos_nf)
    
    # ======= MODIFY COLORBAR WIDTHS =======
    # Make colorbars 70% of the width and align to the right
    colorbar_width_fraction = 1.0
    
    # For colorbar in panel 1
    cb_width1 = pos1.width * colorbar_width_fraction
    cb_x0_1 = pos1.x0 + pos1.width - cb_width1  # Align to right
    new_cb1_pos = [cb_x0_1, pos_cb1.y0, cb_width1, pos_cb1.height]
    ax_cb1.set_position(new_cb1_pos)
    
    # For colorbar in panel 2
    cb_width2 = pos3.width * colorbar_width_fraction
    cb_x0_2 = pos3.x0 + pos3.width - cb_width2  # Align to right
    new_cb2_pos = [cb_x0_2, pos_cb2.y0, cb_width2, pos_cb2.height]
    ax_cb2.set_position(new_cb2_pos)
    
    # Increase spacing between panels 1 and 2
    extra_space = 0.03
    
    # Shift panel 2 and its colorbar to the right
    pos3_updated = ax3.get_position()
    pos_cb2_updated = ax_cb2.get_position()
    
    new_pos3 = [pos3_updated.x0 + extra_space, pos3_updated.y0, pos3_updated.width, pos3_updated.height]
    ax3.set_position(new_pos3)
    
    # Update colorbar position for panel 2 after shifting
    cb_width2 = pos3_updated.width * colorbar_width_fraction
    cb_x0_2 = (pos3_updated.x0 + extra_space) + pos3_updated.width - cb_width2  # Align to right
    new_cb2_pos = [cb_x0_2, pos_cb2_updated.y0, cb_width2, pos_cb2_updated.height]
    ax_cb2.set_position(new_cb2_pos)
    
    # Define custom spacing between plots
    space_between = 0.06  # Smaller value for less space
        
    # Update position after adjustment
    pos3_new = ax3.get_position()
    
    # Adjust space between panels (b) and (c)/(d)
    new_x_pos_c = pos3_new.x0 + pos3_new.width + space_between
    
    # ===== CRITICAL MODIFICATION =====
    # Calculate the bottom position of panel (b)
    panel_b_bottom = pos3_new.y0
    
    # Calculate heights needed for the BZ plot and nesting function
    bz_height = pos_bz.height
    nf_height = pos_nf.height
    
    # Total vertical space needed for both plots
    total_height = bz_height + nf_height + 0.02  # 0.05 for spacing between plots
    
    # Position the nesting function plot to align with the bottom of panel (b)
    ax_nf.set_position([
        new_x_pos_c,
        panel_b_bottom,  # Align bottom with panel (b)
        reference_width,
        nf_height
    ])
    
    # Position the BZ plot above the nesting function plot
    ax_bz.set_position([
        new_x_pos_c -0.005,
        panel_b_bottom + nf_height + 0.05,  # Place above nesting function with spacing
        reference_width,
        bz_height
    ])

# Adjust layout
plt.tight_layout()

# Apply fine-tuning to ensure equal widths and proper spacing
adjust_plot_dimensions()

# Configure colorbar ticks to be inside the colorbar and closer to the colorbar
cbar1.ax.tick_params(direction='in', pad=1)
cbar2.ax.tick_params(direction='in', pad=1)

label1_text = ax_cb1.text(0.5, 2.50, r'$|g_{\bf{q}\nu}|\,\mathrm{(meV)}$', 
                          transform=ax_cb1.transAxes, 
                          fontsize=10, 
                          va='center', 
                          ha='center')

label2_text = ax_cb2.text(0.5, 2.50, r'$\gamma_{\bf{q}\nu}\,\mathrm{(meV)}$', 
                          transform=ax_cb2.transAxes, 
                          fontsize=10, 
                          va='center', 
                          ha='center')

# Save figures
plt.savefig('Fig2_phonon.png', dpi=300, bbox_inches='tight')
plt.savefig('Fig2_phonon.pdf', bbox_inches='tight')
plt.show()