import matplotlib.pyplot as plt
import numpy as np
import matplotlib.ticker as ticker
from matplotlib.ticker import LogLocator

# Set font parameters
plt.rcParams.update({
    'font.family': 'sans-serif',
    'font.sans-serif': ['Arial', 'DejaVu Sans', 'Helvetica', 'Bitstream Vera Sans', 'sans-serif'],
    'font.size': 12,  
    'xtick.labelsize': 12,
    'ytick.labelsize': 12,
    'axes.linewidth': 1.0,
    'xtick.major.width': 1.0,
    'ytick.major.width': 1.0,
    'legend.fontsize': 10,
    'legend.columnspacing': 0.5,
    'legend.handletextpad': 0.25,
    'legend.borderaxespad': 0.25,
})

# Create a figure with two subplots stacked vertically
fig, (ax1, ax2) = plt.subplots(2, 1, figsize=(6, 8))


ax1.set_ylabel(r'$\sigma_{xx}$ (S/cm)')
ax2.set_xlabel('Temperature (K)')
ax2.set_ylabel(r'$\rho_{xx}$ (Ω·cm) [$\times 10^{-5}$]')

# Set the x and y ranges
ax1.set_xlim(0, 325)
ax1.set_ylim(1e4, 2e6)  # Adjust as needed for your data range
ax2.set_xlim(0, 325)
ax2.set_ylim(0, 5.0)

# Load Cond_vs_Temp_140.dat file
try:  
    # Define a function to parse this specific file format
    def parse_cond_file(file_content):
        lines = file_content.strip().split('\n')
        # Skip the header row
        data_lines = lines[1:]
        
        # Initialize lists to store data
        temp = []
        values = [[] for _ in range(13)]  # 13 columns of data values
        
        for line in data_lines:
            # Remove extra spaces and split by comma
            parts = [part.strip() for part in line.split(',')]
            
            # Extract temperature
            temp.append(float(parts[0]))
            
            # Extract values for each column
            for i in range(1, min(len(parts), 14)):
                try:
                    # Handle scientific notation (e.g., 0.429516E+06)
                    values[i-1].append(float(parts[i]))
                except ValueError:
                    # Skip if conversion fails
                    values[i-1].append(np.nan)
        
        # Create a numpy array with temp as the first column
        result = np.column_stack([temp] + values)
        return result
    
    # Use the function to parse the file
    with open('Cond_vs_Temp_140.dat', 'r') as f:
        file_content = f.read()
    file140 = parse_cond_file(file_content)
except Exception as e:
    print(f"Error loading file140: {e}")

# Load exp_huang.dat file
try:
    # This file has a header row and comma-separated values
    with open('exp_huang.dat', 'r') as f:
        lines = f.readlines()
    # Skip the header line, parse the data
    data_lines = lines[1:]
    huang_data = []
    
    for line in data_lines:
        parts = line.strip().split(',')
        if len(parts) >= 2:
            try:
                x = float(parts[0])
                y = float(parts[1])
                # Only include data up to 300K
                if x <= 300:
                    huang_data.append([x, y])
            except ValueError:
                pass
    
    exp_huang = np.array(huang_data)
    print(f"Successfully loaded exp_huang, shape: {exp_huang.shape}")
except Exception as e:
    print(f"Error loading exp_huang: {e}")

# Load the combined Zhang data file
try:
    # Parse the combined data file
    with open('Zhang_data_selected.dat', 'r') as f:
        lines = f.readlines()
    
    # Skip header if present
    start_line = 1 if not any(c.isdigit() for c in lines[0]) else 0
    
    # Initialize arrays for each dataset
    zhang_d1_data = []
    zhang_d11_data = []
    zhang_s1_data = []
    
    # Parse data from the combined file
    for line in lines[start_line:]:
        parts = [p.strip() for p in line.split('\t') if p.strip()]
        if len(parts) >= 6:
            try:
                t_d1 = float(parts[0])
                r_d1 = float(parts[1])
                t_d11 = float(parts[2])
                r_d11 = float(parts[3])
                t_s1 = float(parts[4])
                r_s1 = float(parts[5])
                
                zhang_d1_data.append([t_d1, r_d1])
                zhang_d11_data.append([t_d11, r_d11])
                zhang_s1_data.append([t_s1, r_s1])
            except ValueError:
                pass
    
    # Convert to numpy arrays
    zhang_d1_data = np.array(zhang_d1_data)
    zhang_d11_data = np.array(zhang_d11_data)
    zhang_s1_data = np.array(zhang_s1_data)
except Exception as e:
    print(f"Error loading Zhang data from combined file: {e}")

# Define common plotting parameters
columns = [7, 6, 5, 3, 2]  
colors = ['blue', 'blue', '#FF8000', 'red', 'red']  
column_names = ['−25 meV', '−10 meV', '   0   meV', '+10 meV', '+25 meV']  
markers = ['o', 's', '^', 's', 'o']  
linewidths = [1.5, 1.5, 2.0, 1.5, 1.5]
zorders = [1, 1, 2, 1, 1]  
marker_sizes = [4.0, 3.0, 9, 3.0, 4.0]  

# Define Zhang colors and markers with variations
zhang_colors = ['#707070', '#707070', '#707070']  
zhang_markers = ['s', 'D', '^'] 
zhang_marker_size = 5  

# Filter data to only include points up to 300K
max_temp_idx = np.where(file140[:, 0] <= 300)[0]

# --- PLOT 1: CONDUCTIVITY (TOP PANEL) --- WITH LOG SCALE
if file140.shape[1] >= max(columns) + 1:
    # Plot all datasets in the new reordered sequence
    for i, (col, marker, color, name) in enumerate(zip(columns, markers, colors, column_names)):
        ax1.plot(file140[max_temp_idx, 0], file140[max_temp_idx, col], 
                marker=marker, 
                linestyle='-',
                markersize=marker_sizes[i],
                linewidth=linewidths[i],
                color=color,
                label=name,
                zorder=zorders[i])

# Plot Huang experimental data (conductivity)
if exp_huang.shape[0] > 0:
    ax1.plot(exp_huang[:, 0], exp_huang[:, 1], 
            marker='D',  # Diamond marker
            linestyle='',
            markersize=5,
            color='black',
            markerfacecolor='white',  
            markeredgecolor='black',
            markeredgewidth=1.5,
            label='Huang')

# Plot Zhang data (conductivity) - now using the pre-processed data
# D1 data
ax1.plot(zhang_d1_data[:, 0], 1.0/zhang_d1_data[:, 1], 
        linestyle='',
        color=zhang_colors[0],
        marker=zhang_markers[0],
        markerfacecolor='white',
        markeredgecolor=zhang_colors[0],
        markersize=zhang_marker_size,
        label='Zhang [D1]')

# D11 data
ax1.plot(zhang_d11_data[:, 0], 1.0/zhang_d11_data[:, 1], 
        linestyle='',
        color=zhang_colors[1],
        marker=zhang_markers[1],
        markerfacecolor='white',
        markeredgecolor=zhang_colors[1],
        markersize=zhang_marker_size,
        label='Zhang [D11]')

# S1 data
ax1.plot(zhang_s1_data[:, 0], 1.0/zhang_s1_data[:, 1], 
        linestyle='',
        color=zhang_colors[2],
        marker=zhang_markers[2],
        markerfacecolor='white',
        markeredgecolor=zhang_colors[2],
        markersize=zhang_marker_size,
        label='Zhang [S1]')

# --- PLOT 2: RESISTIVITY (BOTTOM PANEL) ---
if file140.shape[1] >= max(columns) + 1:
    # Plot all datasets in the new reordered sequence
    for i, (col, marker, color, name) in enumerate(zip(columns, markers, colors, column_names)):
        # Calculate resistivity (1/conductivity) and scale to 10^-5
        resistivity = (1.0 / file140[max_temp_idx, col]) * 1e5
        ax2.plot(file140[max_temp_idx, 0], resistivity, 
                marker=marker, 
                linestyle='-',
                markersize=marker_sizes[i],
                linewidth=linewidths[i],
                color=color,
                label=name,
                zorder=zorders[i])

# Plot Huang experimental data (resistivity)
if exp_huang.shape[0] > 0:
    # Calculate resistivity from conductivity and scale to 10^-5
    huang_resistivity = (1.0 / exp_huang[:, 1]) * 1e5
    ax2.plot(exp_huang[:, 0], huang_resistivity, 
            marker='D',  # Diamond marker
            linestyle='',
            markersize=5,
            color='black',
            markerfacecolor='white',
            markeredgecolor='black',
            markeredgewidth=1.5,
            label='Huang')

# Plot Zhang data (resistivity) - now using the pre-processed data
# D1 data - scale to 10^-5
ax2.plot(zhang_d1_data[:, 0], zhang_d1_data[:, 1] * 1e5, 
        linestyle='',
        color=zhang_colors[0],
        marker=zhang_markers[0],
        markerfacecolor='white',
        markeredgecolor=zhang_colors[0],
        markersize=zhang_marker_size)

# D11 data - scale to 10^-5
ax2.plot(zhang_d11_data[:, 0], zhang_d11_data[:, 1] * 1e5, 
        linestyle='',
        color=zhang_colors[1],
        marker=zhang_markers[1],
        markerfacecolor='white',
        markeredgecolor=zhang_colors[1],
        markersize=zhang_marker_size)

# S1 data - scale to 10^-5
ax2.plot(zhang_s1_data[:, 0], zhang_s1_data[:, 1] * 1e5, 
        linestyle='',
        color=zhang_colors[2],
        marker=zhang_markers[2],
        markerfacecolor='white',
        markeredgecolor=zhang_colors[2],
        markersize=zhang_marker_size)

# Configure formatting for both plots
for ax, label in [(ax1, '(a)'), (ax2, '(b)')]:
    # Format axes
    if ax == ax1:  # Conductivity - LOG SCALE
        ax.set_yscale('log')  
        ax1.set_xticklabels([])
        # Use a log locator for y-ticks
        ax.yaxis.set_major_locator(LogLocator(base=10, numticks=6))
        # Format y-tick labels to show 10^ notation instead of scientific notation
        def log_tick_formatter(y, pos):
            if y == 0:
                return '0'
            exponent = int(np.log10(y))
            if exponent == 0:
                return '{:.0f}'.format(y)
            else:
                return r'$10^{{{:d}}}$'.format(exponent)
    else:  # Resistivity - use linear scale as before
        ax.set_yscale('linear')
        ax.set_yticks(np.arange(0, 5.1, 1.0))  
        ax.yaxis.set_minor_locator(ticker.AutoMinorLocator(2))
        
    # X-axis ticks
    ax.set_xticks(np.arange(0, 301, 50))
    ax.xaxis.set_minor_locator(ticker.AutoMinorLocator(2))    
    ax.text(-0.12, 1.02, label, transform=ax.transAxes, 
            fontsize=14, va='top', ha='left')

# Add legend only to the top plot (conductivity) with two columns
handles, labels = ax1.get_legend_handles_labels()
ax1.legend(handles, labels, loc='upper right', ncol=2, fontsize=10, frameon=False, 
           framealpha=0.9, columnspacing=1.0, handletextpad=0.5)

# Adjust layout with more vertical space between subplots
plt.subplots_adjust(hspace=0.10)

# Save figure with high resolution
plt.savefig('Fig3_cond_res.png', dpi=300, bbox_inches='tight')
plt.savefig('Fig3_cond_res.pdf', bbox_inches='tight')
plt.show()