import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import matplotlib.image as mpimg
from matplotlib.ticker import FixedLocator, MultipleLocator, AutoMinorLocator
from matplotlib import rcParams

# Set common font parameters for all plots
plt.rcParams.update({
    'font.family': 'sans-serif',
    'font.sans-serif': ['DejaVu Sans', 'Helvetica', 'Bitstream Vera Sans', 'Arial', 'sans-serif'],
    'font.size': 10,
    'xtick.labelsize': 10,
    'ytick.labelsize': 10,
    'axes.linewidth': 0.8,
    'xtick.major.width': 1.0,
    'ytick.major.width': 1.0,
    'legend.fontsize': 9,
    'legend.columnspacing': 0.7,
    'legend.handletextpad': 0.5,
    'legend.borderaxespad': 0.5,
})

# Create a figure 
fig = plt.figure(figsize=(12, 2.6))
ax_bz = fig.add_axes([0.05, 0.05, 0.15, 0.9])  # For the BZ-doped image
ax1 = fig.add_axes([0.26, 0.2, 0.28, 0.7])    # For the nesting function plot - small gap added
ax2 = fig.add_axes([0.61, 0.2, 0.28, 0.7])    # For the scatter rate plot

# Add labels (a), (b), and (c) positioned outside the plots
ax_bz.text(-0.1, 0.92, '(a)', transform=ax_bz.transAxes, fontsize=12)
ax1.text(-0.2, 0.96, '(b)', transform=ax1.transAxes, fontsize=12)
ax2.text(-0.2, 0.96, '(c)', transform=ax2.transAxes, fontsize=12)

# =============================================================
# LEFT PLOT (BZ IMAGE) - load BZ-doped.png
# =============================================================
# Load the BZ-doped.png image
try:
    bz_img = mpimg.imread('BZ-doped.png')
    # Turn off axes completely to remove all space
    ax_bz.axis('off')  
    # Display the image without any padding or spacing
    ax_bz.imshow(bz_img, aspect='equal')
except FileNotFoundError:
    ax_bz.text(0.5, 0.5, 'BZ-doped.png not found', 
               horizontalalignment='center', verticalalignment='center',
               transform=ax_bz.transAxes)

# =============================================================
# MIDDLE PLOT (NESTING FUNCTION) - from plot_nesting.py
# =============================================================

# Define y-axis range for nesting plot
ymin_nesting = 0.0
ymax_nesting = 1.5

# Define x positions for the high-symmetry points
x1 = 0.0000  # Gamma
x2 = 0.9984  # Sigma
x3 = 1.2808  # N
x4 = 1.5632  # Sigma_1
x5 = 2.4018  # Z
x6 = 2.9435  # Gamma
x7 = 4.2424  # X

# Function to read data
def read_data(filename):
    data = np.loadtxt(filename, skiprows=1)
    return data

# Read data from nesting file
try:
    nesting_data = read_data("nf_ndd_dgs2.dat")

    # Add vertical lines at high-symmetry points
    for x in [x2, x3, x4, x5, x6]:
        ax1.axvline(x=x, color='black', linestyle='-', linewidth=0.2, alpha=0.5, zorder=0)

    # Set x-ticks at high-symmetry points with Greek symbols
    x_positions = [x1, x2, x3, x4, x5, x6, x7]
    x_labels = [r'$\Gamma$', r'$\Sigma$', 'N', r'$\Sigma_1$', 'Z', r'$\Gamma$', 'X']
    ax1.set_xticks(x_positions)
    ax1.set_xticklabels(x_labels)

    # Set y-ticks and minor ticks for nesting plot
    ax1.set_ylim(0, 1.5)
    ax1.set_yticks(np.arange(0, 1.6, 0.5))  # Major ticks at 0.0, 0.5, 1.0, 1.5
    ax1.yaxis.set_minor_locator(AutoMinorLocator(2))

    # Set minor tick parameters
    ax1.tick_params(axis='y', which='minor', length=4)  # Length of minor ticks
    ax1.tick_params(axis='y', which='major', length=7)  # Length of major ticks

    # Set labels for nesting plot
    ax1.set_ylabel(r'$\zeta_q$ (eV$^{-2}$)')

    # Plot the nesting data
    ax1.plot(
        nesting_data[:,0],
        nesting_data[:,1]/185.096025,
        color='blue',
        linewidth=1.0, 
        label="−25 meV"
    )
    
    ax1.plot(
        nesting_data[:,0],
        nesting_data[:,3]/185.096025,
        color='black',
        linestyle='--',
        linewidth=1.0,
        label="   0   meV"
    )

    ax1.plot(
            nesting_data[:,0],
            nesting_data[:,5]/185.096025,
            color='red',
            linestyle='-',
            linewidth=1.0, 
            label="+25 meV"
    )

    ax1.set_xlim(x1, x7)

    ax1.legend(loc='upper left', frameon=False)
except FileNotFoundError:
    ax1.text(0.5, 0.5, 'nf_ndd_dgs2.dat not found', 
             horizontalalignment='center', verticalalignment='center',
             transform=ax1.transAxes)

# =============================================================
# RIGHT PLOT (SCATTER RATE) - from plot_SR.py
# =============================================================

# Define file paths for scatter rate data
SR0 = "p_SR0.dat"
SR25 = "p_SR25.dat"
SR_25 = "p_SR-25.dat"
avg = "avg.dat"

# Function to read data from files using pandas
def read_pandas_data(filename):
    try:
        return pd.read_csv(filename, sep=r'\s+', header=None)
    except Exception as e:
        print(f"Error reading {filename}: {e}")
        return None

# Read data from scatter rate files
try:
    data_SR0 = read_pandas_data(SR0)
    data_SR25 = read_pandas_data(SR25)
    data_SR_25 = read_pandas_data(SR_25)
    data_avg = read_pandas_data(avg)

    # Set plot range and labels for scatter rate plot
    ax2.set_ylim(0, 40)  # Adjusted for THz scale (25 meV ≈ 6 THz)
    ax2.set_xlim(-0.1, 0.1)
    ax2.set_xticks([-0.1, -0.05, 0, 0.05, 0.1])

    # Add minor ticks to x-axis - 2 minor ticks between major ticks
    ax2.xaxis.set_minor_locator(AutoMinorLocator(2))

    # Add minor ticks to y-axis - 2 minor ticks between major ticks
    ax2.yaxis.set_minor_locator(AutoMinorLocator(2))
    ax2.axvline(x=0, color='black', linestyle='--', linewidth=0.5, zorder=0)

    # Configure tick appearance
    ax2.tick_params(axis='both', which='minor', length=4)  # Length of minor ticks
    ax2.tick_params(axis='both', which='major', length=7)  # Length of major ticks

    # Set labels for scatter rate plot
    ax2.set_xlabel(r"$\varepsilon - \varepsilon_{\rm F}$ (eV)")
    ax2.set_ylabel(r"$\tau_{n\bf{k}}^{-1}$ (THz)")  # Changed unit to THz

    # Plot the scatter points with conversion to THz
    ax2.scatter(
        data_SR_25[0]*13.605-17.0383,
        data_SR_25[6]*20670.6944033,
        color='blue',
        alpha=0.5,
        s=4,
        edgecolors='none',
        label="−25 meV"
    )

    ax2.scatter(
        data_SR0[0]*13.605-17.0633,
        data_SR0[6]*20670.6944033, 
        color='black',
        alpha=0.5,
        s=4,
        edgecolors='none',  
        label="   0   meV"
    )

    ax2.scatter(
        data_SR25[0]*13.605-17.0883,
        data_SR25[6]*20670.6944033,  # Convert to THz
        color='red',
        alpha=0.5,
        s=4,
        edgecolors='none',
        label="+25 meV"
    )

    # Plot the lines from avg file with conversion to THz
    ax2.plot(
        data_avg[0],
        data_avg[3]*20670.6944033,  # Convert to THz
        color='red',
        linewidth=1.5,
        linestyle='-'
    )

    ax2.plot(
        data_avg[0],
        data_avg[1]*20670.6944033,  # Convert to THz
        color='black',
        linewidth=1.5,
        linestyle='-'
    )
        
    ax2.plot(
        data_avg[0],
        data_avg[2]*20670.6944033,  # Convert to THz
        color='blue',
        linewidth=1.5,
        linestyle='-'
    )

    # Set legend for scatter rate plot
    ax2.legend(loc='upper left', frameon=False, markerscale=2.2)
except FileNotFoundError:
    ax2.text(0.5, 0.5, 'Scatter rate data files not found', 
             horizontalalignment='center', verticalalignment='center',
             transform=ax2.transAxes)

ax_bz.set_aspect('auto')
ax_bz.margins(0, 0)


plt.savefig('Fig4_SR.png', dpi=300, bbox_inches='tight')
plt.savefig('Fig4_SR.pdf', bbox_inches='tight')

plt.show()