import matplotlib.pyplot as plt
import numpy as np
import matplotlib.colors as colors
from matplotlib.gridspec import GridSpec
from matplotlib import ticker
from matplotlib.collections import LineCollection

# Set font parameters 
plt.rcParams.update({
    'backend': 'TkAgg',
    'font.family': 'sans-serif',
    'font.sans-serif': ['DejaVu Sans', 'Helvetica', 'Bitstream Vera Sans', 'Arial', 'sans-serif'],
    'font.size': 12,
    'xtick.labelsize': 12,
    'ytick.labelsize': 12,
    'axes.linewidth': 0.8,
    'xtick.major.width': 1.0,
    'ytick.major.width': 1.0,
    'legend.columnspacing': 0.9,
    'legend.handletextpad': 0.25,
    'legend.borderaxespad': 0.25,
})

# Function to process and reshape DFT data
def process_bands(band_file):
    print(f"Reading band data from: {band_file}")
    data = np.loadtxt(band_file)
    k = np.unique(data[:, 0])
    num_bands = len(data) // len(k)
    bands = np.reshape(data[:, 1], (num_bands, len(k)))
    weight = np.reshape(data[:, 2], (num_bands, len(k)))
    return k, bands, weight

# Create figure with one row containing three plots
fig = plt.figure(figsize=(10, 2.8))

band_dos_gs = GridSpec(1, 2, width_ratios=[0.75, 0.25], wspace=0.08, 
                     left=0.1, right=0.53, figure=fig)
fermi_gs = GridSpec(1, 1, left=0.62, right=0.98, figure=fig)

# Create the axes with specific positioning
ax_band = fig.add_subplot(band_dos_gs[0, 0]) 
ax_dos = fig.add_subplot(band_dos_gs[0, 1])   
ax_fermi = fig.add_subplot(fermi_gs[0, 0])    

# Load band structure data
k, bands, weight = process_bands('fatband_TaAs.dat')
print(f"Band structure data loaded: {bands.shape[0]} bands")

# Define x-coordinates for high-symmetry points
x1 = 0.0000
x2 = 0.9984
x3 = 1.2808
x4 = 1.5632
x5 = 2.4018
x6 = 2.9435
x7 = 4.2424
xmin_band = 0.0
xmax_band = 4.2424
ymin_band = -0.2
ymax_band = 0.2
Ef = 17.0633  # Fermi energy

# Band structure plot settings
ax_band.set_xlim(xmin_band, xmax_band)
ax_band.set_ylim(ymin_band, ymax_band)
ax_band.tick_params(axis='y', which='both', left=True, labelleft=True)

# Add vertical lines at high-symmetry points
for x in [x2, x3, x4, x5, x6]:
    ax_band.axvline(x=x, color='black', linestyle='-', linewidth=0.2, alpha=0.5, zorder=0)

# Add horizontal line at zero energy
ax_band.axhline(y=0, color='black', linestyle='--', linewidth=1, alpha=0.5, zorder=0)
ax_band.axhline(y=0.025, color='red', linestyle='dotted', linewidth=1, alpha=0.8, zorder=0)
ax_band.axhline(y=-0.025, color='blue', linestyle='dotted', linewidth=1, alpha=0.8, zorder=0)

# Set x-ticks at high-symmetry points
ax_band.set_xticks([xmin_band, x2, x3, x4, x5, x6, xmax_band])
ax_band.set_xticklabels([r'$\Gamma$', r'$\Sigma$', 'N', r'$\Sigma_1$', 'Z', r'$\Gamma$', 'X'])
ax_band.yaxis.set_minor_locator(ticker.AutoMinorLocator(4))
ax_band.set_yticks([-0.2, -0.1, 0, 0.1, 0.2])

# Set y-label
ax_band.set_ylabel(r'$\varepsilon - \varepsilon_{\rm F}^0$ (eV)')

# Plot bands with simple black color
for i in range(bands.shape[0]):
    ax_band.plot(k, bands[i], color='k', linewidth=1.5)

ax_band.text(-0.2, 1.0, '(a)', transform=ax_band.transAxes, fontsize=12, 
             ha='center', va='center')

# Load DOS data
total_dos = np.loadtxt('dos_0.25.dat')

# DOS plot settings
ax_dos.set_xlim(0, 0.4)
ax_dos.set_ylim(ymin_band, ymax_band)
ax_dos.set_xlabel('DOS \n (States/eV)')
ax_dos.set_xticks([0, 0.4])
ax_dos.set_xticklabels(['0', '0.4'])

# Set the minor ticks for DOS plot
ax_dos.yaxis.set_minor_locator(ticker.AutoMinorLocator(4))
ax_dos.xaxis.set_minor_locator(ticker.AutoMinorLocator(4))
ax_dos.axhline(y=0, color='black', linestyle='--', linewidth=1, alpha=0.5, zorder=0)
ax_dos.axhline(y=0.025, color='red', linestyle='dotted', linewidth=1, alpha=0.8, zorder=0)
ax_dos.axhline(y=-0.025, color='blue', linestyle='dotted', linewidth=1, alpha=0.8, zorder=0)

# Remove y-ticks from DOS plot but keep the same range as band plot
ax_dos.set_yticklabels([])
ax_dos.set_yticks(ax_band.get_yticks())

# Plot DOS data
ax_dos.plot(total_dos[:, 1], total_dos[:, 0], 'k-', linewidth=1.0)
ax_dos.fill_betweenx(total_dos[:, 0], 0, total_dos[:, 1], color='gray', alpha=0.15)

Ef = 17.0633  # Fermi energy
ymin = -0.5
ymax = 10.25
xmin = 15
xmax = 125

# Load data
filename = "Ef_vs_nk.dat"
data = np.loadtxt(filename, skiprows=1)  # Skip header
print(f"Fermi energy convergence data loaded from {filename}")

# Plot data with line and points
ax_fermi.plot(data[:, 0], (data[:, 1] - Ef) * 1000, 'k-', marker='o', markersize=5, 
         linewidth=1.5, markerfacecolor='black', markeredgecolor='black')

ax_fermi.axhline(y=0, color='black', linestyle='--', linewidth=1)

ax_fermi.set_xlim(xmin, xmax)
ax_fermi.set_ylim(ymin, ymax)

ax_fermi.set_xlabel("k-grid")
ax_fermi.set_ylabel(r"$\varepsilon_F - \varepsilon_F^{0}$ (meV)")

x_ticks = np.arange(20, 121, 20)
ax_fermi.set_xticks(x_ticks)
x_tick_labels = [f"{x}$^3$" for x in x_ticks]
ax_fermi.set_xticklabels(x_tick_labels)

# Configure y-ticks
ax_fermi.set_yticks(np.arange(0, 11, 5))
ax_fermi.yaxis.set_minor_locator(ticker.AutoMinorLocator(4))  
ax_fermi.xaxis.set_minor_locator(ticker.AutoMinorLocator(2))  

ax_fermi.text(-0.15, 1.0, '(b)', transform=ax_fermi.transAxes, fontsize=12, 
             ha='center', va='center')

plt.tight_layout()

plt.savefig('FigS1_band_ef.png', dpi=300, bbox_inches='tight')
plt.savefig('FigS1_band_ef.pdf', bbox_inches='tight')

plt.show()