import numpy as np
import matplotlib.pyplot as plt
import matplotlib.ticker as ticker
from matplotlib.legend_handler import HandlerTuple
import matplotlib.lines as mlines

# Set font parameters - matching paste.txt
plt.rcParams.update({
    'font.family': 'sans-serif',
    'font.sans-serif': ['DejaVu Sans', 'Helvetica', 'Bitstream Vera Sans', 'Arial', 'sans-serif'],
    'font.size': 12,  
    'xtick.labelsize': 12,
    'ytick.labelsize': 12,
    'axes.linewidth': 0.8,
    'xtick.major.width': 1.0,
    'ytick.major.width': 1.0,
    'legend.columnspacing': 0.5,
    'legend.handletextpad': 0.25,
    'legend.borderaxespad': 0.25,
    'legend.fontsize': 9.5,
})

# Read the grid convergence data file
def read_conductivity_data(filename):
    with open(filename, 'r') as f:
        lines = f.readlines()
    
    # Parse the header to get grid sizes
    headers = lines[0].strip().split()
    grid_sizes = headers[1:]
    
   
    temperatures = []
    conductivity_data = {}
    
    for grid in grid_sizes:
        conductivity_data[grid] = []
    
    for line in lines[1:]:
        values = line.strip().split()
        if len(values) > 1:  
            temperatures.append(float(values[0]))
            for i, grid in enumerate(grid_sizes):
                conductivity_data[grid].append(float(values[i+1]))
    
    return temperatures, conductivity_data, grid_sizes

# Function to read SERTA-BTE comparison data
def read_serta_bte_data(filename):
    try:
        data = np.loadtxt(filename, skiprows=2, max_rows=8)
        print(f"Successfully loaded {filename}")
        return data
    except (FileNotFoundError, ValueError) as e:
        print(f"Warning: {filename} not found or couldn't be properly read. Error: {e}")
        return None

# Function to read CRTA data
def read_crta_data(filename):
    try:
        with open(filename, 'r') as f:
            lines = f.readlines()
        
        headers = lines[0].strip().split()
        
        temperatures = []
        crta_data = {}
        
        for header in headers:
            crta_data[header] = []
        
        # Parse data
        for line in lines[1:]:
            values = line.strip().split()
            if len(values) > 1:  
                temperatures.append(float(values[0]))
                for i, header in enumerate(headers):
                    if i < len(values) - 1:  # Skip temperature column
                        crta_data[header].append(float(values[i+1]))
        
        return temperatures, crta_data, headers
    except (FileNotFoundError, ValueError) as e:
        print(f"Warning: {filename} not found or couldn't be properly read. Error: {e}")
        return None, None, None

fig, (ax1, ax2) = plt.subplots(1, 2, figsize=(10, 4.2))  
plt.subplots_adjust(wspace=0.3) 

# ===== SUBPLOT (a): Grid convergence plot =====
filename = 'Cond_vs_T_mesh.dat'
try:
    temperatures, conductivity_data, grid_sizes = read_conductivity_data(filename)
    
    valid_temps = [t for t in temperatures if 50 <= t <= 300]
    temps_indices = [temperatures.index(t) for t in valid_temps]
    
    filtered_temps = valid_temps
    filtered_data = {}
    for grid in grid_sizes:
        filtered_data[grid] = [conductivity_data[grid][i] for i in temps_indices]

    markers = ['o', 's', '^', 'd', 'x']  
    colors = ['blue', 'red', 'green', 'purple', 'orange']

    for i, grid in enumerate(grid_sizes):
        if i < len(markers) and i < len(colors):
            ax1.plot(filtered_temps, filtered_data[grid], 
                    marker=markers[i], 
                    markersize=5, 
                    linestyle='-', 
                    linewidth=1.5, 
                    color=colors[i], alpha=0.8,
                    label=grid)
except FileNotFoundError:
    print(f"Warning: {filename} not found")
    ax1.text(0.5, 0.5, 'Grid convergence data not available', 
             horizontalalignment='center', verticalalignment='center',
             transform=ax1.transAxes)

ax1.set_yscale('log')
ax1.set_ylim(5e3, 1.5e6) 

y_ticks = [1e4, 1e5, 1e6]
ax1.set_yticks(y_ticks)

# Format the tick labels with scientific notation
def scientific_formatter(x, pos):
    if x == 0:
        return '0'
    else:
        return r'$10^{' + '{:.0f}'.format(np.log10(x)) + '}$'

ax1.yaxis.set_major_formatter(ticker.FuncFormatter(scientific_formatter))
ax1.yaxis.set_minor_locator(ticker.LogLocator(base=10.0, subs=np.arange(2, 10) * 0.1, numticks=10))

ax1.set_xlim(40, 310)
ax1.set_xticks(np.arange(50, 301, 50))
ax1.xaxis.set_minor_locator(ticker.AutoMinorLocator(2))


ax1.set_xlabel('Temperature (K)')
ax1.set_ylabel(r'$\sigma_{xx}$ (S/cm)')

ax1.tick_params(axis='both', which='both', direction='in')
ax1.tick_params(which='major', length=6)
ax1.tick_params(which='minor', length=3)


if 'filtered_data' in locals():
    ax1.legend(loc='upper right', frameon=False)


ax1.text(-0.15, 0.96, '(a)', transform=ax1.transAxes, fontsize=12)

# ===== SUBPLOT (b): CRTA-SERTA-BTE comparison =====
file1 = "SERTA_vs_BTE_cond.dat"  # 0 meV
file2 = "SERTA_vs_BTE_cond25.dat"  # 25 meV
file3 = "SERTA_vs_BTE_cond-25.dat"  # -25 meV
crta_file = "crta.dat"  # CRTA data

# Load SERTA-BTE data from files
data1 = read_serta_bte_data(file1)
data2 = read_serta_bte_data(file2)
data3 = read_serta_bte_data(file3)

crta_temps, crta_data, crta_headers = read_crta_data(crta_file)

color_neg25 = 'blue'
color_0 = 'black'
color_25 = 'red'

# Plot CRTA data with square markers and dotted lines
if crta_temps is not None and crta_data is not None:
    crta_neg25_key = "-25meV-CRTA"
    crta_0_key = "0meV-CRTA"
    crta_25_key = "25meV-CRTA"

    scaling_factor = 1e5  
    
    # Plot CRTA data with lower zorder to avoid covering other plots
    if crta_neg25_key in crta_data:
        crta_neg25 = ax2.plot(crta_temps, np.array(crta_data[crta_neg25_key])*scaling_factor, 's:', 
                             color=color_neg25, alpha=0.7, markersize=4, linewidth=1.5, label='-25 meV', zorder=15)[0]
    if crta_0_key in crta_data:
        crta_0 = ax2.plot(crta_temps, np.array(crta_data[crta_0_key])*scaling_factor, 's:', 
                         color=color_0, alpha=0.7, markersize=4, linewidth=1.5, label='0 meV', zorder=15)[0]
    if crta_25_key in crta_data:
        crta_25 = ax2.plot(crta_temps, np.array(crta_data[crta_25_key])*scaling_factor, 's:', 
                          color=color_25, alpha=0.7, markersize=4, linewidth=1.5, label='25 meV', zorder=15)[0]

# Plot data with SERTA lines (solid) - increased zorder
if data1 is not None:
    serta_0 = ax2.plot(data1[:,0], data1[:,1]*1e5, 'o-', color=color_0, alpha=0.7, markersize=5, linewidth=1.5, label='0 meV', zorder=10)[0]
if data2 is not None:
    serta_25 = ax2.plot(data2[:,0], data2[:,1]*1e5, 'o-', color=color_25, alpha=0.7, markersize=5, linewidth=1.5, label='25 meV', zorder=10)[0]
if data3 is not None:
    serta_neg25 = ax2.plot(data3[:,0], data3[:,1]*1e5, 'o-', color=color_neg25, alpha=0.7, markersize=5, linewidth=1.5, label='-25 meV', zorder=10)[0]

# Plot BTE lines (dashed) with diamond markers - highest zorder
if data1 is not None:
    bte_0 = ax2.plot(data1[:,0], data1[:,2]*1e5, 'D--', color=color_0, alpha=0.7, markersize=8, mfc='white', markeredgewidth=1.5, linewidth=1.5, label='0 meV', zorder=5)[0]
if data2 is not None:
    bte_25 = ax2.plot(data2[:,0], data2[:,2]*1e5, 'D--', color=color_25, alpha=0.7, markersize=8, mfc='white', markeredgewidth=1.5, linewidth=1.5, label='25 meV', zorder=5)[0]
if data3 is not None:
    bte_neg25 = ax2.plot(data3[:,0], data3[:,2]*1e5, 'D--', color=color_neg25, alpha=0.7, markersize=8, mfc='white', markeredgewidth=1.5, linewidth=1.5, label='-25 meV', zorder=5)[0]

ax2.set_xlim(40, 310)
ax2.set_xticks(np.arange(50, 301, 50))
ax2.xaxis.set_minor_locator(ticker.AutoMinorLocator(2))

ax2.set_yscale('log')
ax2.set_ylim(5e3, 1.5e6)  
ax2.set_yticks(y_ticks)

legend_elements = [
    # First column header - CRTA
    mlines.Line2D([], [], color='white', marker='', linestyle='', label='CRTA'),
    # CRTA items - blue square with dotted line
    mlines.Line2D([], [], color=color_neg25, marker='s', linestyle=':', markersize=4, label='−25 meV'),
    # CRTA items - black square with dotted line
    mlines.Line2D([], [], color=color_0, marker='s', linestyle=':', markersize=4, label='   0   meV'),
    # CRTA items - red square with dotted line
    mlines.Line2D([], [], color=color_25, marker='s', linestyle=':', markersize=4, label='+25 meV'),
    
    # Second column header - SERTA
    mlines.Line2D([], [], color='white', marker='', linestyle='', label='SERTA'),
    # SERTA items - blue circle with solid line
    mlines.Line2D([], [], color=color_neg25, marker='o', linestyle='-', markersize=5, label='−25 meV'),
    # SERTA items - black circle with solid line
    mlines.Line2D([], [], color=color_0, marker='o', linestyle='-', markersize=5, label='   0   meV'),
    # SERTA items - red circle with solid line
    mlines.Line2D([], [], color=color_25, marker='o', linestyle='-', markersize=5, label='+25 meV'),
    
    # Third column header - BTE
    mlines.Line2D([], [], color='white', marker='', linestyle='', label='BTE'),
    # BTE items - blue diamond with dashed line
    mlines.Line2D([], [], color=color_neg25, marker='D', mfc='white', linestyle='--', markersize=7, label='−25 meV'),
    # BTE items - black diamond with dashed line
    mlines.Line2D([], [], color=color_0, marker='D', mfc='white', linestyle='--', markersize=7, label='   0   meV'),
    # BTE items - red diamond with dashed line
    mlines.Line2D([], [], color=color_25, marker='D', mfc='white', linestyle='--', markersize=7, label='+25 meV'),
]

ax2.legend(handles=legend_elements, loc='upper right',  
ncol=3, frameon=False, handletextpad=0.5, columnspacing=1.0)

ax2.yaxis.set_major_formatter(ticker.FuncFormatter(scientific_formatter))
ax2.yaxis.set_minor_locator(ticker.LogLocator(base=10.0, subs=np.arange(2, 10) * 0.1, numticks=10))

ax2.tick_params(axis='both', which='both', direction='in')
ax2.tick_params(which='major', length=6)
ax2.tick_params(which='minor', length=3)

ax2.set_xlabel('Temperature (K)')
ax2.set_ylabel(r'$\sigma_{xx}$ (S/cm)')

ax2.text(-0.15, 0.96, '(b)', transform=ax2.transAxes, fontsize=12)

plt.tight_layout(rect=[0, 0, 1, 0.85])

plt.savefig('FigS3_Conv_cond.png', dpi=300, bbox_inches='tight')
plt.savefig('FigS3_Conv_cond.pdf', bbox_inches='tight')

plt.show()