import numpy as np
import matplotlib.pyplot as plt
import matplotlib.ticker as ticker

# Set font parameters
plt.rcParams.update({
    'font.family': 'sans-serif',
    'font.sans-serif': ['DejaVu Sans', 'Helvetica', 'Bitstream Vera Sans', 'Arial', 'sans-serif'],
    'font.size': 10,  
    'xtick.labelsize': 10,
    'ytick.labelsize': 10,
    'axes.linewidth': 0.8,
    'xtick.major.width': 1.0,
    'ytick.major.width': 1.0,
    'legend.columnspacing': 0.5,
    'legend.handletextpad': 0.25,
    'legend.borderaxespad': 0.5,
    'legend.fontsize': 10,
})


fileCRTA2 = "crta_Ef_T300.dat"

# Function to read data files
def read_data(filename):
    try:
        data = np.loadtxt(filename, skiprows=1)  
        return data
    except FileNotFoundError:
        print(f"Warning: {filename} not found")

data = read_data(fileCRTA2)

fig, ax = plt.subplots(figsize=(4.5, 3.0))

crta_line = ax.plot(data[:, 0], data[:, 1]*1e4, 'ro-',
                   markersize=5, linewidth=1.5,
                   label='CRTA',  zorder=10)[0]

bte_line = ax.plot(data[:, 0], data[:, 2]*1e4, 'kD-',
                  markersize=5, mfc='white', markeredgewidth=1.5, linewidth=1.5,
                  label='BTE',  zorder=5)[0]

ax.set_yscale('log')
ax.set_ylim(2e4, 5e4)

ax.set_xlim(-30, 30)
ax.set_xticks(np.arange(-30, 31, 10))
ax.xaxis.set_minor_locator(ticker.AutoMinorLocator(2))
y_ticks =  [2e4, 5e4]
ax.set_yticks(y_ticks)

# Format the tick labels 
def scientific_formatter(x, pos):
    if x == 0:
        return '0'
    else:
        return r'$10^{' + '{:.0f}'.format(np.log10(x)) + '}$'

ax.yaxis.set_major_formatter(ticker.FuncFormatter(scientific_formatter))
ax.yaxis.set_minor_locator(ticker.LogLocator(base=10.0, subs=np.arange(2, 10) * 0.1, numticks=10))
ax.tick_params(axis='both', which='both', direction='in')
ax.tick_params(which='major', length=6)
ax.tick_params(which='minor', length=3)

ax.set_xlabel(r'$\varepsilon_F - \varepsilon_F^0$ (meV)')
ax.set_ylabel(r'$\sigma_{xx}$ (S/cm)')

ax.legend(loc='upper right', frameon=False)

plt.tight_layout()

plt.savefig('FigS4_CRTA.png', dpi=300, bbox_inches='tight')
plt.savefig('FigS4_CRTA.pdf', bbox_inches='tight')

plt.show()